import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:mobx/mobx.dart';
import 'package:share_plus/share_plus.dart';
import 'dart:developer';
import 'package:vesti_logger/logger/vlog.dart';

part 'vlogger_store.g.dart';

class VLoggerStore = _VLoggerStore with _$VLoggerStore;

abstract class _VLoggerStore with Store {
  @observable
  bool showTopIndicator = false;

  @observable
  bool showBottomIndicator = false;

  @observable
  bool isEnabled = false;

  @observable
  bool isCopying = false;

  @observable
  String filter = "";

  @observable
  bool isDebug = false;

  @observable
  bool isError = false;

  @observable
  bool isInfo = false;

  @computed
  ObservableList<VLog> get filteredLogs => _logFiltered();

  @observable
  ObservableList<VLog> logs = ObservableList();

  _VLoggerStore();

  reset() {
    isDebug = false;
    isError = false;
    isInfo = false;
    filter = "";
  }

  @action
  toggle() {
    isEnabled = !isEnabled;
  }

  @action
  atTop(bool isAtTop) {
    showBottomIndicator = !isAtTop;
    showTopIndicator = isAtTop;
  }

  @action
  copyRequests() {
    _copyLogs(VLogType.Debug);
  }

  @action
  copyResponses() {
    _copyLogs(VLogType.Info);
  }

  @action
  copyErrors() {
    _copyLogs(VLogType.Error);
  }

  _copyLogs(VLogType logType) {
    String copiedRequests = "";
    logs
        .where((log) => log.type == logType)
        .toList()
        .asMap()
        .forEach((key, value) {
      copiedRequests +=
          "${value.message ?? ""}${key == logs.length - 1 ? "" : "\n\n"}";
    });
    if (copiedRequests.isNotEmpty) {
      Share.share(copiedRequests);
    } else {
      Fluttertoast.showToast(
        timeInSecForIosWeb: 4,
        msg: "Nenhum dado para ser compartilhado no momento",
        backgroundColor: Colors.black87,
        gravity: ToastGravity.TOP,
      );
    }
  }

  debug({String? title, String? message}) {
    _log(title: title, message: message, type: VLogType.Debug);
  }

  error({String? title, String? message}) {
    _log(title: title, message: message, type: VLogType.Error);
  }

  info({String? title, String? message}) {
    _log(title: title, message: message, type: VLogType.Info);
  }

  _log({String? title, String? message, VLogType type = VLogType.Debug}) {
    String updatedTitle = "$title\n";
    log(type.separator +
        "\n" +
        updatedTitle +
        (message ?? "") +
        "\n" +
        type.separator);
    logs.add(VLog(title: title, message: message, type: type));
  }

  ObservableList<VLog> _logFiltered() {
    List<VLog> filteredTypeLogs = _logTypeFiltered();
    List<VLog> filteredLogs = filteredTypeLogs.where((log) {
      var title = log.title ?? "";
      var message = log.message ?? "";
      return title.toLowerCase().contains(filter) ||
          message.toLowerCase().contains(filter);
    }).toList().reversed.toList();
    return ObservableList<VLog>.of(filteredLogs);
  }

  List<VLog> _logTypeFiltered() {
    List<VLog> filteredTypeLogs = [];
    if (isDebug == false && isError == false && isInfo == false) {
      filteredTypeLogs = logs.toList();
      return filteredTypeLogs;
    }
    if (isDebug)
      filteredTypeLogs.addAll(logs.where((log) => log.type == VLogType.Debug));

    if (isError)
      filteredTypeLogs.addAll(logs.where((log) => log.type == VLogType.Error));

    if (isInfo)
      filteredTypeLogs.addAll(logs.where((log) => log.type == VLogType.Info));

    return filteredTypeLogs;
  }
}
